// ----------------------------------------------------
/*
   class CWorld

   Author :   Sylvain Lefebvre
   Created:   06/11/2002

 */
// ----------------------------------------------------
#ifndef __CWORLD_H_
#define __CWORLD_H_
// ----------------------------------------------------
#include <list>
#include <vector>
// ----------------------------------------------------
#include "CMobile.h"
#include "CNetwork.h"
#include "INetworkGame.h"
#define Pi 3.1415927
void display_error(const string msg, int mob=0, int ligne=0, int col=0);
void display_warning(const string msg, int mob=0, int ligne=0, int col=0);
void display_error_clear();
// ----------------------------------------------------
#define WORLD_SCALE          100.0
#define WORLD                (CWorld::getUniqueInstance())
#define WORLD_BORDER_RADIUS  WORLD_SCALE
#define WORLD_BORDER_BOTTOM  0
#define WORLD_BORDER_RIGHT   1
#define WORLD_BORDER_TOP     2
#define WORLD_BORDER_LEFT    3
// ----------------------------------------------------
#define KEY_LEFT  1
#define KEY_RIGHT 2
#define KEY_UP    3
#define KEY_DOWN  4
#define KEY_SPACE 5
// ----------------------------------------------------
#define WORLD_MSG_RESET NG_MSG_USER+1
// ----------------------------------------------------
class CWorld : public INetworkGame
{
private:
  
  std::list<CMobile *>                m_Mobiles;
  std::vector<CMobile *>              m_Locals;
  std::vector<std::vector<CMobile*> > m_Distants;
  int                                 m_iClic;
  int                                 m_iUnClic;
  int                                 m_iClicked;
  bool                                key[256];
  double                              t, dt;
  double                              m_tDispError;
  
  CWorld();

public:

  ~CWorld();
  static CWorld& getUniqueInstance();

  CMobile              *addMobile();
  void                  addMobile(CMobile *m);
  void                  addMobileToLocal(CMobile *m);
  void                  removeMobile(CMobile *m);
  void                  step();
  std::list<CMobile *>& getMobiles() {return (m_Mobiles);}
  CMobile              *getLocalMobile(int n);
  CMobile              *getDistantMobile(int m,int c);
  CMobile              *getMobile(int m,int c);
  CMobile              *getPrevMobile(CMobile*);
  CMobile              *getNextMobile(CMobile*);
  CMobile              *getNearest(CMobile *, int p=1);
  CMobile              *getNearestDir(CMobile *,double dx,double dy,double a=Pi,int p=1);
  int                   getLocalMobileSize() const {return (m_Locals.size());}
  int                   getMobileLocalId(CMobile *);
  void                  sampleTime();
  double                getTime() { return t; };
  double                getTimeInterval() { return dt; };
  void                  resetTime();
  void                  reset();
  void                  resetClient(int);
  void                  drawString(const char *);
  void                  drawMobileString(CMobile *,const char *);
  void                  drawNames();
  void                  drawFrame();
  bool                  drawState();
  void                  drawMobiles();
  void                  setClic(int c)    { m_iClic    = c; };
  void                  setUnClic(int c)  { m_iUnClic  = c; };
  void                  setClicked(int c) { m_iClicked = c; };
  int                   getClic()   { return m_iClic;    };
  int                   getUnClic() { return m_iUnClic;  };
  int                   getClicked(){ return m_iClicked; };
  void                  setKey(int k) { key[k] = true; };
  void                  resetKeys() { for(int i=0; i<256; i++) key[i]=false; };
  bool                  isKeyPressed(int k) { return key[k]; };
  CMobile              *getMouse();
  CMobile              *getCamera();
  CMobile              *getLight1();
  CMobile              *getLight2();
  CMobile              *getLight3();

  // from INetworkGame
  virtual IGhost *createGhost(int t,int id,
			      const unsigned char *d,int l);
  virtual void    onNewGhost(IGhost *);
  virtual void    onNetworkReady();
  virtual void    onAllGhostsReceived(int);
  virtual void    onClientConnection(int);
  virtual void    onClientDisconnection(int);
  virtual void    processClientMessage(const network_msg&);

  // iterator on mobiles
  typedef std::list<CMobile *>::iterator       it_mobile;

};
// ----------------------------------------------------
#endif
// ----------------------------------------------------
